package com.boylesoftware.cb2.examples.addressbook;

import java.util.Map;
import java.util.HashMap;

import com.boylesoftware.cb2.BLObject;
import com.boylesoftware.cb2.DAO;
import com.boylesoftware.cb2.BLErrors;
import com.boylesoftware.cb2.BLException;


/**
 * BLO that represents the address book business entity.
 *
 * @author Lev Himmelfarb
 * @version $Id: AddressBookBLO.java,v 1.1.1.1 2004/04/04 18:51:45 levahim Rel $
 */
public class AddressBookBLO
    extends BLObject {

    /**
     * Error code indicating that a person with the same
     * first and last name already exists.
     */
    public static final int ERROR_NAME_EXISTS = 1;


    /**
     * Searches the database for person records matching a certain
     * condition.
     *
     * @param lastNameSubstr substring of a person's last
     * name. <code>null</code> if last name should not participate in
     * the filter.
     * @param firstNameSubstr substring of a person's first name or
     * <code>null</code>.
     * @param citySubstr substring of the city name in a person's home
     * or business address or <code>null</code>.
     * @param state two-letter US state code in a person's home or
     * business address or <code>null</code>.
     *
     * @return array of descriptors of records matching the condition
     * or an empty array.
     *
     * @throws BLException if a database error happens.
     */
    public PersonShortDM [] searchPeople(String lastNameSubstr,
                                         String firstNameSubstr,
                                         String citySubstr,
                                         String state)
        throws BLException {

        // prepare query parameters and conditions
        Map params = new HashMap(4);
        if(lastNameSubstr != null)
            params.put("lastName", "%" + lastNameSubstr.trim().toUpperCase() + "%");
        if(firstNameSubstr != null)
            params.put("firstName", "%" + firstNameSubstr.trim().toUpperCase() + "%");
        if(citySubstr != null)
            params.put("city", "%" + citySubstr.trim().toUpperCase() + "%");
        if(state != null)
            params.put("state", state.trim().toUpperCase());

        // call the search query and return the results
        return (PersonShortDM [])this.getDAO().
            fetchWithNamedParams("searchPeople",
                                 params.keySet(),
                                 params);
    }

    /**
     * Gets a person record details by the person id.
     *
     * @param personId id of the record requested.
     *
     * @return completely filled person DM with home and work
     * addresses and all phone numbers.
     *
     * @throws BLException if a database error happens or no record
     * with the specified id exists.
     */
    public PersonDM getPersonDetails(int personId)
        throws BLException {

        // fetch the person record
        PersonDM [] res = (PersonDM [])this.getDAO().
            fetch("fetchPersonDetailsById",
                  new Object [] {new Integer(personId)});

        // check if the record was found
        if(res.length < 1)
            throw new BLException("Could not find requested person record with id [" +
                                  personId + "].");

        // done, return person details
        return res[0];
    }

    /**
     * Creates new or updates existing person record basing in the
     * information provided in the specified DM. If
     * <code>personId</code> in the DM is equal or less than zero a
     * new record is created, otherwise an existing record with that
     * id is updated.
     *
     * @param person a DM with the new data including home and work
     * address and phone numbers nested DMs.
     *
     * @return <code>true</code> if the operation was successful,
     * <code>false</code> if there the operation cannot be performed,
     * for example, beacuse another record with the same first and
     * last names exist. Check <code>getLastErrors()</code> if the
     * method returns <code>false</code>.
     *
     * @throws BLException if a database error happens or trying to
     * update a record with <code>personId</code> which does not
     * exist.
     */
    public boolean savePersonDetails(PersonDM person)
        throws BLException {

        // get the DAO
        DAO dao = this.getDAO();

        // create personId wrapper, we will need it multiple times later
        Integer personId = new Integer(person.personId);

        // check if a record with the same first and last names
        // but different personId exists
        Map params = new HashMap(3);
        // include first name check if specified
        if(person.firstName != null)
            params.put("firstName", person.firstName.toUpperCase());
        params.put("lastName", person.lastName.toUpperCase());
        // if it's a new record then personId is invalid and no record
        // exists with the same personId anyway
        params.put("personId", personId);
        // this query selects personIds into PersonShortDM for records
        // with the specified names and different from the specified one personIds
        if(dao.fetchWithNamedParams("checkIfSameNameAndDiffIdExists",
                                    params.keySet(),
                                    params).length > 0) {
            BLErrors errors = new BLErrors(1);
            errors.addError(ERROR_NAME_EXISTS);
            this.setErrors(errors);
            return false;
        }

        // see if we are creating a new record or updating an existing one
        if(person.personId > 0) { // update

            // fetch existing record (will throw BLException if person
            // does not exist)
            PersonDM oldPerson = this.getPersonDetails(person.personId);

            // delete existing phone numbers
            dao.update("deletePhonesByPersonId",
                       new Object [] { personId });

            // update home address
            if((person.homeAddress != null) && (oldPerson.homeAddress != null)) {
                person.homeAddress.addressId = oldPerson.homeAddress.addressId;
                dao.update(person.homeAddress);
                person.homeAddressId = oldPerson.homeAddressId;
            } else if((person.homeAddress != null) && (oldPerson.homeAddress == null)) {
                dao.insert(person.homeAddress);
                person.homeAddressId = new Integer(person.homeAddress.addressId);
            } else if((person.homeAddress == null) && (oldPerson.homeAddress != null)) {
                dao.delete(oldPerson.homeAddress);
                person.homeAddressId = null;
            } else {
                person.homeAddressId = null;
            }

            // update work address
            if((person.workAddress != null) && (oldPerson.workAddress != null)) {
                person.workAddress.addressId = oldPerson.workAddress.addressId;
                dao.update(person.workAddress);
                person.workAddressId = oldPerson.workAddressId;
            } else if((person.workAddress != null) && (oldPerson.workAddress == null)) {
                dao.insert(person.workAddress);
                person.workAddressId = new Integer(person.workAddress.addressId);
            } else if((person.workAddress == null) && (oldPerson.workAddress != null)) {
                dao.delete(oldPerson.workAddress);
                person.workAddressId = null;
            } else {
                person.workAddressId = null;
            }

            // update person record
            dao.update(person);

            // reinsert phone numbers
            if(person.phones != null) {
                for(int i = 0; i < person.phones.length; i++) {
                    person.phones[i].personId = person.personId;
                    dao.insert(person.phones[i]);
                }
            }

        } else { // create new

            // insert home address record
            if(person.homeAddress != null) {
                dao.insert(person.homeAddress);
                person.homeAddressId = new Integer(person.homeAddress.addressId);
            } else
                person.homeAddressId = null;

            // insert work address record
            if(person.workAddress != null) {
                dao.insert(person.workAddress);
                person.workAddressId = new Integer(person.workAddress.addressId);
            } else
                person.workAddressId = null;

            // insert person record
            dao.insert(person);

            // insert phone records
            if(person.phones != null) {
                for(int i = 0; i < person.phones.length; i++) {
                    person.phones[i].personId = person.personId;
                    dao.insert(person.phones[i]);
                }
            }
        }

        // all done, report success
        return true;
    }

    /**
     * Deletes a person record from the database.
     *
     * @param personId id of the record to delete.
     *
     * @return DM with data of the just deleted contact.
     *
     * @throws BLException if a database error happens or no record
     * with the specified id exist.
     */
    public PersonDM deletePerson(int personId)
        throws BLException {

        // get the DAO
        DAO dao = this.getDAO();

        // fetch the current record
        PersonDM dm = this.getPersonDetails(personId);

        // delete phones
        if(dm.phones.length > 0)
            dao.update("deletePhonesByPersonId",
                       new Object [] { new Integer(personId) });

        // delete addresses
        AddressDM adDm = new AddressDM();
        if(dm.homeAddressId != null) {
            adDm.addressId = dm.homeAddressId.intValue();
            dao.delete(adDm);
        }
        if(dm.workAddressId != null) {
            adDm.addressId = dm.workAddressId.intValue();
            dao.delete(adDm);
        }

        // call delete and check if the record was deleted
        if(dao.delete(dm) < 1)
            throw new BLException("Could not find requested person record with id [" +
                                  personId + "].");

        // done, return the DM with old data
        return dm;
    }
}
