package com.boylesoftware.cb2.examples.addressbook.presentation;

import com.boylesoftware.cb2.BLException;
import com.boylesoftware.cb2.presentation.servlet.PresentationElement;
import com.boylesoftware.cb2.presentation.servlet.ActionContext;
import com.boylesoftware.cb2.util.TextUtils;

import com.boylesoftware.cb2.examples.addressbook.AddressBookBLO;
import com.boylesoftware.cb2.examples.addressbook.PersonShortDM;


/**
 * This presentation element performs contact search and is used in
 * the component to show the resulting contacts list. It expects a
 * <code>ContactsSearchFormPE</code> at its input.
 *
 * @author Lev Himmelfarb
 * @version $Id: ContactsListPE.java,v 1.1.1.1 2004/04/04 18:51:45 levahim Rel $
 */
public final class ContactsListPE
    implements PresentationElement {


    ///////////////////////////////////////////////////////////////////////////
    // Inner classes
    ///////////////////////////////////////////////////////////////////////////

    /**
     * Descriptor of a record in the contacts list.
     */
    public static class Contact {

        /**
         * Person id.
         */
        private final String personId;

        /**
         * Sequencial number of this record.
         */
        private final String seqN;

        /**
         * Person full name in form "Last Name, First Name".
         */
        private final String name;


        /**
         * Gets person id for the record.
         *
         * @return the person id.
         */
        public String getPersonId() {

            return this.personId;
        }

        /**
         * Gets sequencial number of this record.
         *
         * @return sequencial number of this record.
         */
        public String getSeqN() {

            return this.seqN;
        }

        /**
         * Gets person full name for the record.
         *
         * @return person full name.
         */
        public String getName() {

            return this.name;
        }


        /**
         * Create a descriptor.
         *
         * @param dm person DM.
         */
        Contact(int seqN, PersonShortDM dm) {

            this.personId = String.valueOf(dm.personId);
            this.seqN = String.valueOf(seqN);
            this.name = dm.lastName + (dm.firstName != null ? ", " + dm.firstName : "");
        }
    }


    ///////////////////////////////////////////////////////////////////////////
    // Presentation element data
    ///////////////////////////////////////////////////////////////////////////

    /**
     * List of contacts, the search result.
     */
    private Contact [] contacts;


    /**
     * Gets the list of contacts to display in the component.
     *
     * @return the list of contacts.
     */
    public Contact [] getContacts() {

        return this.contacts;
    }

    /**
     * Gets the length of the list of contacts. If used in the
     * component to determine if the search returned any results.
     *
     * @return the length of the list of contacts.
     */
    public int getContactsLength() {

        return this.contacts.length;
    }


    ///////////////////////////////////////////////////////////////////////////
    // Presentation element interface
    ///////////////////////////////////////////////////////////////////////////

    /**
     * Initialize presentation element.
     *
     * @param actionCtx the action context.
     *
     * @throws BLException if there was an unexpected error in the
     * business level.
     */
    public void init(ActionContext actionCtx)
        throws BLException {

        // find the BLO
        AddressBookBLO abBlo = (AddressBookBLO)actionCtx.getBLO("addressBook");

        // fetch contact records
        PersonShortDM [] dms;
        ContactsSearchFormPE form = (ContactsSearchFormPE)actionCtx.getForm();
        if(form.getDoSearch()) {
            dms = abBlo.searchPeople(TextUtils.getNullIfEmpty(form.getLastName()),
                                     TextUtils.getNullIfEmpty(form.getFirstName()),
                                     TextUtils.getNullIfEmpty(form.getCity()),
                                     (TextUtils.isEmpty(form.getState()) ? null : form.getState().trim().toUpperCase()));
        } else {
            dms = abBlo.searchPeople(null, null, null, null);
        }

        // populate the list of found contacts
        this.contacts = new Contact[dms.length];
        for(int i = 0; i < dms.length; i++)
            this.contacts[i] = new Contact(i + 1, dms[i]);
    }

    /**
     * Reset presentation element's internal fields. This method is
     * not used in this class because the presentation element
     * represented by this class has no input parameters.
     */
    public void reset() {}
}
